// AutoCAD Runtime Extension (ARX), 11/12/04
// This program is only for testing purposes
// (arxload "Test")

#include <aced.h> // acedRegCmds
#include <geassign.h> // asDblArray
#include <dbents.h> // AcDbBlockTableRecord
#include <dbapserv.h> // acdbHostApplicationServices
#include <adslib.h> // RTSTR

#define pi 3.14159265358979323846

// AutoLisp Functions (external ARX subroutines, EXRXSUBR): http://xarch.tu-graz.ac.at/autocad/code/vnestr/c_printf.c
int f() {
 struct resbuf *args, *rbp;
 args=acedGetArgs();
 rbp = args;
 switch(rbp->restype) { // while(rbp) // (f 'a): invalid data type or data overflow
  case RTVOID:    acedRetVoid();                    break;
  case RTNIL:     acedRetNil();                     break; // nil, char*
  case RTT:       acedRetT();                       break; // T
  case RTSHORT:   acedRetInt(rbp->resval.rint);     break; // int: -32768 to 32767
  case RTLONG:    acedRetInt(rbp->resval.rlong);    break; // long: 32768 to 2147483647, (also -2147483648 to -32769)
  case RTREAL:    acedRetReal(rbp->resval.rreal);   break; // double
  case RTPOINT:   acedRetPoint(rbp->resval.rpoint); break; // '(0.0 0.0)
  case RT3DPOINT: acedRetPoint(rbp->resval.rpoint); break; // '(0.0 0.0 0.0)
  case RTSTR:     acedRetStr(rbp->resval.rstring);  break; // str, char*
//case RTENAME:   acedRetName(rbp->resval.rlname,*);break; // ent, long
//case RTPICKS:   acedRetName(rbp->resval.rlname,*);break; // ss, long
//case RTDOTE:                                      break; // dotted pair
//case RTLB:                                        break; // list begin
//case RTLE:                                        break; // list end
//case RTRESBUF:                                    break;
 }
//rbp=rbp->rbnext;
 return 0;
}
int listExample() { // http://www.cadvault.com/forums/archive/index.php/t-2078.html
 struct resbuf *tlist;
 tlist = acutBuildList(RTREAL, 1.0, RTSTR, "Hello", RTLB, RTREAL, pi, RTLE, 0);
 acedRetList(tlist); // acedCmd(tlist);
 acutRelRb(tlist);
 return 0;
}
int type2() { // http://www.icadweb.com/doc/help/dr/IDR_SDS_B332.htm
 struct resbuf *args, *rbp; // linked list: sds_resbuf
 args=acedGetArgs();
 rbp = args;
 switch(rbp->restype) { // while(rbp)
  case RTVOID:    acedRetStr("void");        break; // Unhandled Exception 77C3F0E3 (Access Violation Reading 0x0004) at address 2911EC9h
  case RTNIL:     acedRetStr("nil");         break;
  case RTT:       acedRetStr("T");           break;
  case RTSHORT:   acedRetStr("short");       break;
  case RTLONG:    acedRetStr("long");        break;
  case RTREAL:    acedRetStr("real");        break;
  case RTANG:     acedRetStr("angle");       break; // ?
  case RTPOINT:   acedRetStr("point");       break;
  case RT3DPOINT: acedRetStr("3Dpoint");     break;
  case RTSTR:     acedRetStr("str");         break;
  case RTDXF0:    acedRetStr("DXF code 0");  break; // ?
  case RTDOTE:    acedRetStr("dotted pair"); break; // doesn't work
  case RTLB:      acedRetStr("list begin");  break;
  case RTLE:      acedRetStr("list end");    break; // RTNONE
  case RTENAME:   acedRetStr("ent");         break;
  case RTPICKS:   acedRetStr("ss");          break;
  case RTRESBUF:  acedRetStr("result buffer list"); break;
  default:        acedRetStr("unknown");     break;
// invalid data type or data overflow (*not tested yet): SYM SUBR *USUBR EXRXSUBR FILE *VLR-Object-Reactor *VLA-OBJECT *variant *safearray
 }
 return 0;
}

// Input Functions
void inputFunctions() {
 int i;
 double x, theta, d;
 char str[256]; // 133
 AcGePoint3d p;
 ads_point p2;
 ads_name ent; // ss;
 acedInitGet(RSG_NONEG | RSG_NOZERO, ""); // initialize acedGet functions

 // Command Line Input
 acedGetInt("\nEnter an integer: ", &i);
 acedGetReal("\nEnter a real number: ", &x);
 acedGetString(0, "\nEnter a string: ", str); // 0?
//acedGetKword("\nEnter a keyword: ", str); // key word
 acutPrintf("\nYou entered: %d, %lf, ", i, x); // 1.000000
 acutPrintf(str);

 // Points
 acedGetPoint(NULL, "\nSpecify first point: ", asDblArray(p));
 acedGetAngle(asDblArray(p), "\nSpecify second point: ", &theta);
 acutPrintf("\nThe angle is %lf radians.", theta);
 acedGetDist(asDblArray(p), "\nSpecify second point: ", &d);
 acutPrintf("\nThe distance is %lf.", d);
 acedGetCorner(asDblArray(p), "\nSpecify other corner: ", p2);

 // Entities
 acedEntSel("\nSelect object: ", ent, p2);
 acutPrintf("\nYou selected entity: %ld", ent[0]); // ID
}

// Draw Entities
void entmake(AcDbEntity *ent) {
 AcDbBlockTableRecord *record;
 acdbOpenObject(record, acdbHostApplicationServices()->workingDatabase()->currentSpaceId(), AcDb::kForWrite);
 record->appendAcDbEntity(ent);
 record->close();
 ent->close();
}
void point(AcGePoint3d p) {
 AcDbPoint *ent = new AcDbPoint();
 ent->setDatabaseDefaults();
 ent->setPosition(p);
 entmake(ent);
}
void line(AcGePoint3d p1, AcGePoint3d p2) {
 AcDbLine *ent = new AcDbLine();
 ent->setDatabaseDefaults();
 ent->setStartPoint(p1);
 ent->setEndPoint(p2);
//ent->setColorIndex(1);
 entmake(ent);
}
void circle(AcGePoint3d p, double r) {
 AcDbCircle *ent = new AcDbCircle;
 ent->setDatabaseDefaults();
 ent->setCenter(p);
 ent->setRadius(r);
 entmake(ent);
}
void arc(AcGePoint3d p, double r, double theta1, double theta2) {
 AcDbArc *ent = new AcDbArc;
 ent->setDatabaseDefaults();
 ent->setCenter(p);
 ent->setRadius(r);
 ent->setStartAngle(theta1);
 ent->setEndAngle(theta2);
 entmake(ent);
}
void drawEntities() {
 AcGePoint3d p = AcGePoint3d(0,0,0);
 point(AcGePoint3d(0,0,0));
 line(AcGePoint3d(0,0,0), AcGePoint3d(1,1,0));
 circle(AcGePoint3d(0,0,0), 1);
 arc(AcGePoint3d(0,0,0), 2, 0, pi);
}

struct fdata {char *name; int (*pointer)();}; // function data
fdata functions[] = {
 {"f", f},
 {"listExample", listExample},
 {"type2", type2},
 {0, 0}
};
struct cdata {char *name; void (*pointer)();}; // command data
cdata commands[] = {
 {"inputFunctions", inputFunctions},
 {"drawEntities", drawEntities},
 {0, 0}
};
void initApp() {int i; // called by AutoCAD when application loads
 for(i=0; functions[i].name!=0; i++) { // Register AutoLisp Functions
  acedDefun(functions[i].name,i);
  acedRegFunc(functions[i].pointer,i);
 }
 for(i=0; commands[i].name!=0; i++) { // Register Commands with AutoCAD command mechanism
  acedRegCmds->addCommand("TEST_COMMANDS", commands[i].name, commands[i].name, ACRX_CMD_TRANSPARENT, commands[i].pointer);
 }
 acutPrintf("\nCommands: inputFunctions, drawEntities"); // printf redirected to AutoCAD command line
 acutPrintf("\nFunctions: f, listExample, type2");
}

void unloadApp() {acedRegCmds->removeGroup("TEST_COMMANDS");}

extern "C" AcRx::AppRetCode
acrxEntryPoint(AcRx::AppMsgCode msg, void* pkt) { // msg: ObjectARX message to application, pkt: opaque data handle
 switch (msg) {
  case AcRx::kInitAppMsg:
   acrxDynamicLinker->unlockApplication(pkt); // default applications are locked: cannot be unloaded
   acrxDynamicLinker->registerAppMDIAware(pkt); // Multiple Document Interface (MDI)
   initApp();
  break;
  case AcRx::kUnloadAppMsg: unloadApp(); break;
 }
 return AcRx::kRetOK;
}
